@@ -39,11 +39,10 @@ def get_kaptive_paths():
 def get_kaptive_results(locus_type, kaptive_py, kaptive_db, contigs, args):
     assert locus_type == 'K' or locus_type == 'O'
 
-    headers = ['K_locus', 'K_locus_confidence', 'K_locus_problems', 'K_locus_identity',
+    headers = ['K_locus', 'K_type', 'K_locus_confidence', 'K_locus_problems', 'K_locus_identity',
                'K_locus_missing_genes']
     if locus_type == 'O':
-        headers = [x.replace('K_locus', 'O_locus') for x in headers]
-        headers.append('O_type')
+        headers = [x.replace('K_', 'O_') for x in headers]
 
     if (args.kaptive_k and locus_type == 'K') or (args.kaptive_o and locus_type == 'O'):
         if locus_type == 'K':
@@ -51,12 +50,13 @@ def get_kaptive_results(locus_type, kaptive_py, kaptive_db, contigs, args):
         else:  # locus_type == 'O':
             outfile = args.kaptive_o_outfile
 
-        kaptive_results = run_kaptive(kaptive_py, kaptive_db, contigs, outfile, one_thread=False)
+        # Try with multiple threads, and if that fails (possible due to a multithreading BLAST
+        # error), try again with one thread.
+        kaptive_results = run_kaptive(kaptive_py, kaptive_db, contigs, outfile,
+                                      args.min_kaptive_confidence, one_thread=False)
         if kaptive_results is None:
-            kaptive_results = run_kaptive(kaptive_py, kaptive_db, contigs, outfile, one_thread=True)
-        if locus_type == 'O':
-            o_locus = kaptive_results[0]
-            kaptive_results.append(get_o_type(o_locus))
+            kaptive_results = run_kaptive(kaptive_py, kaptive_db, contigs, outfile,
+                                          args.min_kaptive_confidence, one_thread=True)
 
         assert len(headers) == len(kaptive_results)
         return dict(zip(headers, kaptive_results))
@@ -64,7 +64,7 @@ def get_kaptive_results(locus_type, kaptive_py, kaptive_db, contigs, args):
         return {}
 
 
-def run_kaptive(kaptive_py, kaptive_db, contigs, output_file, one_thread):
+def run_kaptive(kaptive_py, kaptive_db, contigs, output_file, min_confidence, one_thread):
     thread_option = ' --threads 1' if one_thread else ''
     with tempfile.TemporaryDirectory() as tmp_dir:
         kaptive_prefix = tmp_dir + '/kaptive'
@@ -92,11 +92,12 @@ def run_kaptive(kaptive_py, kaptive_db, contigs, output_file, one_thread):
 
         if p.returncode != 0:
             if stderr:
-                sys.exit('Error: Kaptive failed to run with the following error:\n' + stderr.strip())
+                sys.exit('Error: Kaptive failed to run with the following error:\n' +
+                         stderr.strip())
             else:
                 sys.exit('Error: Kaptive failed to run')
 
-        locus, confidence, problems, identity = None, None, None, None
+        locus, serotype, confidence, problems, identity = None, None, None, None, None
         missing = []
 
         # Parse the required information from the Kaptive verbose output.
@@ -105,6 +106,8 @@ def run_kaptive(kaptive_py, kaptive_db, contigs, output_file, one_thread):
         for line in output_lines:
             if 'Best match locus:' in line:
                 locus = line.split('Best match locus:')[1].strip()
+            if 'Best match type:' in line:
+                serotype = line.split('Best match type:')[1].strip()
             if 'Match confidence:' in line:
                 confidence = line.split('Match confidence:')[1].strip()
             if 'Problems:' in line:
@@ -132,23 +135,21 @@ def run_kaptive(kaptive_py, kaptive_db, contigs, output_file, one_thread):
             with open(output_file, 'at') as f:
                 f.write(kaptive_table_lines[1])		 # write data line
 
-    if locus is None or confidence is None or problems is None or identity is None:
+    if locus is None or serotype is None or confidence is None or problems is None or identity is None:
         sys.exit('Error: Kaptive failed to produce the expected output')
 
-    return [locus, confidence, problems, identity, ','.join(missing)]
+    if not confidence_meets_threshold(confidence, min_confidence):
+        locus = f'unknown ({locus})'
+        if not serotype.startswith('unknown'):
+            serotype = f'unknown ({serotype})'
+
+    return [locus, serotype, confidence, problems, identity, ','.join(missing)]
 
 
-def get_o_type(o_locus):
+def confidence_meets_threshold(confidence, min_confidence):
     """
-    This function returns an O type using the O locus. In many cases, they are the same, except for:
-    * loci O1v1 and O1v2 = type O1
-    * loci O2v1 and O2v2 = type O2
-    * loci O1/O2v1 and O1/O2v2 = either O1/O2 or 'unknown' (thoughts?)
+    Returns True if the confidence level meets or exceeds the minimum confidence level.
     """
-    if 'O1/O2' in o_locus:
-        return 'unknown'
-    if 'v1' in o_locus:
-        return o_locus.replace('v1', '')
-    if 'v2' in o_locus:
-        return o_locus.replace('v2', '')
-    return o_locus
+    min_confidence = min_confidence.replace('_', ' ')
+    scores = {'None': 0, 'Low': 1, 'Good': 2, 'High': 3, 'Very high': 4, 'Perfect': 5}
+    return scores[confidence] >= scores[min_confidence]
